package kit

import (
	"github.com/labstack/echo"
)

type Auther struct {
	TokenName string
	funcs     map[string]echo.MiddlewareFunc
}

type AutherFunc func(token string) (user interface{}, err error)

func NewAuther(tokenName string) *Auther {
	return &Auther{
		TokenName: tokenName,
		funcs:     make(map[string]echo.MiddlewareFunc),
	}
}

func (auth *Auther) Registry(name string, fn AutherFunc) {
	auth.funcs[name] = auth.echoMiddlewareWrap(fn)
}

func (auth *Auther) Delegate(name string) echo.MiddlewareFunc {
	return auth.funcs[name]
}

func (auth *Auther) echoMiddlewareWrap(fn AutherFunc) echo.MiddlewareFunc {
	return func(next echo.HandlerFunc) echo.HandlerFunc {
		return func(c echo.Context) error {
			if auth.TokenName == "" || fn == nil {
				return next(c)
			}

			token := auth.findAuthToken(c)
			if token == "" {
				return next(c)
			}

			user, err := fn(token)
			if err != nil {
				panic(err)
			}

			// save valid token for later use
			c.Set("token", token)
			c.Set("user", user)
			return next(c)
		}
	}
}

func (auth *Auther) findAuthToken(c echo.Context) (token string) {
	// cookie
	cookie, _ := c.Request().Cookie(auth.TokenName)
	if cookie != nil {
		token = cookie.Value
		return
	}

	// header
	token = c.Request().Header.Get(auth.TokenName)
	if token != "" {
		return
	}

	// query
	token = c.QueryParam(auth.TokenName)
	if token != "" {
		return
	}

	return
}
