package kit

import (
	"gopher/config"
	"io"
	"os"
	"path"

	UserObject "gopher/ak/orm/user"

	"github.com/labstack/echo"
	"github.com/labstack/gommon/log"
	"github.com/opisok/gopkg/token"
)

var (
	UserTokenStore token.TokenStore
	LogKit         *log.Logger
	AutherKit      *Auther
)

func Init() {
	initAuther()
	initLog()
	initThirdPartServices()
}

func initLog() {
	LogKit = log.New("app")
	LogKit.SetLevel(log.DEBUG)
	if config.GlobalConfig.LogDir == "" {
		LogKit.SetOutput(os.Stdout)
		return
	}
	p := path.Join(config.GlobalConfig.LogDir, "app.log")
	logFile, err := os.OpenFile(p, os.O_CREATE|os.O_APPEND|os.O_WRONLY, 0666)
	if err != nil {
		panic(err)
	}
	LogKit.SetOutput(io.MultiWriter(os.Stdout, logFile))
}

// auther 在请求过程中将回话token转换成一个应用定义的user对象, 放入当前请求上下文, 可通过req.Context.Get("user") 获取
func initAuther() {
	mgoURL := config.GlobalConfig.MongoURL
	tokenName := config.GlobalConfig.TokenName

	// 输入token, 输出user
	UserTokenStore = token.NewTokenAuthOnMongod(mgoURL, "userToken", tokenName)

	userAuth := func(token string) (user interface{}, err error) {
		// uid => user, 通过uid 从数据库或其他服务获取用户信息
		uid, err := UserTokenStore.Get(token)
		if err != nil {
			return
		}
		return UserObject.Get(uid.(int64))
	}

	// 可注册多个认证方法, 实现AutherFunc即可, 每个api只能使用一个
	AutherKit = NewAuther(tokenName)
	AutherKit.Registry("userAuth", userAuth)
}

// 从上下文获取用户帮助函数, 因为上下文中user 是一个interface{}, 所以需要转换
func MustGetUser(ctx echo.Context, err error) (user *UserObject.User) {
	iuser := ctx.Get("user")
	if iuser == nil {
		panic(err)
	}
	return iuser.(*UserObject.User)
}

func TryGetUser(ctx echo.Context) (user *UserObject.User) {
	iuser := ctx.Get("user")
	if iuser == nil {
		return nil
	}
	return iuser.(*UserObject.User)
}

// 初始化第三方服务
func initThirdPartServices() {
}
