package test

import (
	"fmt"
	UserObject "gopher/ak/orm/user"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"

	"github.com/bxcodec/faker"
)

func TestObjectUser(t *testing.T) {
	// Setup
	InitDB("../config/test.yaml", true)

	person := faker.GetPerson()
	email := fmt.Sprintf("%s@xyz.com", person.FirstName())
	name := person.Name()

	u := UserObject.NewUser(email, name)
	err := u.Save()
	assert.NoError(t, err)

	u1, err := UserObject.Get(u.Id)
	assert.NoError(t, err)
	assert.Equal(t, u.Name, u1.Name)
	assert.Equal(t, u.CTime, u1.CTime)

	updater := map[string]interface{}{
		"email": "new@xyz.com",
	}
	time.Sleep(1 * time.Second)
	u2, err := UserObject.Update(u1.Id, updater)
	assert.NoError(t, err)
	assert.Equal(t, u.Name, u2.Name)
	assert.Equal(t, u.CTime, u2.CTime)
	assert.NotEqual(t, u.MTime, u2.MTime)
	assert.Equal(t, u2.Email, "new@xyz.com")

	n, err := UserObject.Query().Count()
	assert.NoError(t, err)
	fmt.Println("user count:", n)

	us, err := UserObject.Query().WithPage(0, 10).All()
	assert.NoError(t, err)
	for _, u := range us {
		fmt.Println(u)
	}
	assert.NoError(t, UserObject.Delete(u2.Id))

	u3, err := UserObject.Get(u2.Id)
	assert.NoError(t, err)
	fmt.Println(u3)
}
