# gop

解析objmodel.txt以及apimodel.txt 自动生成Golang 代码, 实现快速开发

- 服务可抽象为3层, api + logic + object(DB)
- model 的主要作用是通过模板减少代码的编写, 使开发主要集中在logic层
- apimodel.txt 用于描述api层, 生成服务路由,请求参数,参数校验等
- objmodel.txt 用于描述object(DB)层, 生成服务对象数据库操作
- 对象变量名统一使用驼峰风格, Json/Bson 等字符串标签, 首字母小写
- 数据库使用mongodb, 每个对象有一个自增ID, 类型为`int64`


## 目录结构
```
apimodel.txt # 接口定义
objmodel.txt # 对象定义
main.go # 工程入口
config/ # 工程配置定义
app/ # 程序业务逻辑, 开发主要集中在这个目录
app/api/ # 程序业务API逻辑
app/service/ # 程序业务复杂逻辑
app/info/ # 业务自定义对象
ak/ # 管理自动生成的API和Object
ak/gw/ # HTTP接口
ak/gw/doc.go # 接口文档，自动生成
ak/gw/model/ # 请求对象，自动生成
ak/orm/model/model.go # 业务逻辑对象定义，自动生成
ak/orm/id.go # 对象全局自增ID，自动生成
kit/ # 通用组件实例化
test/ # 测试用例
```

## 常用操作
- `pip install m2c` 安装帮助工具
- 根据apimodel.txt, 自动生成API `m2c apicode`
- 根据objmodel.txt, 自动生成Object `m2c objcode`
- 生成openApi swagger, `m2c openapi`
- 单元测试 `m2c unittest`
- 生成接口文档 `m2c apidoc`
- 二进制打包发布 `m2c pack`


## 自动生成对象操作
自动生成Object, 封装mongodb操作
`objmodel.txt -> ak/object`

语法:

```
root object # 开始标记
topObjName TopObjName[] # comment, Top Object
    filedObjName [Prefix]filedObjName[Suffix] [*] # comment Filed Object
```

- Object 按类型分两种, 一种基本类型对象BaseType, 一种自定义类型对象StructType
- Object 按位置分两种, 一种顶级对象TopObject, 一种字段对象FiledObject
- comment 可以为API 文档提供信息

**TopObject**

- 顶级对象, 可以被其他对象引用@ (Ref)
- 一定以`[]` 结尾, 表示一定是一个对象的集合

**FiledObject**

- 字段对象, 作为其他对象的一个字段
- FiledObject 可以为BaseType 也可以为StructType
- 如果为StructType 不会主动序列化
- Prefix 默认为空, 无特殊含义(可以认为为Manager 关系)
- Prefix 为 `^` 时表示序列化时隐藏当前字段
- Prefix 为 `@` 时表示当前是引用一个TopObject, 且不会主动序列化
- Subfix 默认为空, 无特殊含义
- Subfix 为 `[]` 时表示对象集合, 代码生成时会对对象集合生成特定的代码, 如分页, 过滤等等
- 最后的 `*` 也可隐藏字段不主动序列化

**BaseType**

- str 对应string, str[] 对应[]string
- bool 对应bool, bool[] 对应[]bool
- int, float, double 对应int64, x[] 对应[]int64
- dict, jsonNode 对应map[string]interface{}
- any 对应 interface{}

**StructType**

- structType 自定义类型对用Go 的struct 类型
- structType[] List 类型，对应 []structType
- 每个structType 对象自动绑定`id`, `ctime`, `mtime` 3个字段，`id`唯一
- 建议structType 嵌套不要超过3层
- 对应表名为TopObjectName + [StructObjectName]


## 自动生成API和APIDOC
支持`GET Query String` 和`POST Json` 两种风格接口

`apimodel.txt -> ak/gw/model.go -> apidoc`

语法:
```
root apiPrefix
apiGroup # comment
    apiName [method=get] [auth=[none, userAuth, adminAuth]] [proto=[json, raw]] # comment
        reqFiled filedType [required=true] # comment
```

`auth` 表示是否认证, 可自定义, 通过Auther.Registry 注册即可
    - 默认不认证
    - userAuth 普通用户认证
    - adminAuth 管理员认证

`proto` 表示数据协议类型, 目前支持两种
    - 默认json
    - raw 表示原生类型, 即数据类型由业务逻辑自己决定, 可以延伸为binary, xml等

```
method /apiPrefix/apiGroup/apiName
```

type关系如下 modelType -> goType
```
TYPEMAPPING = {
    'str': 'string',
    'string': 'string',
    'int': 'int64',
    'long': 'int64',
    'float': 'int64',
    'double': 'int64',
    'bool': 'bool',
    'any': 'interface{}',
    'dict': 'map[string]interface{}',
    'JsonNode': 'map[string]interface{}',
}
```
List 定义如下: 已`[]` 结尾的为List，如下
```
str[] -> []string
```

### 如何基于`gop` 创建新工程
0. `pip install m2c` 安装帮助工具
1. `mkdir <your-project-name> && cd <your-project-name>` 创建工程目录
2. `m2c init` 初始化工程
3. 定义apimodel.txt 和objmodel.txt
4. `m2c apicode && m2c objcode` 生成apimodel和objmodel
5. 编写业务逻辑

### 如何设置cookie
每个req 对象中包含一个请求的Context(注意在定义api时避免使用context作为变量名)
```
cookie := &http.Cookie{
    Name:    "auth-token",
    Value:   token,
    Expires: time.Now().Add(time.Duration(60*60) * time.Second),
}
req.Context.SetCookie(cookie)
return
```

### 如何验证用户身份
1. 定义认证方法, 实现AuthFunc将回话ID转换成用户信息, 参考`kit.go`
2. apimodle 定义api 时设置认证方法 
3. 请求执行时, 会执行自定义的认证方法, 将用户信息放到当前请求上下文中
4. 通过req.Context.Get("user") 获取用户信息, 参考`kit.go`

### 如何返回内容
当前返回内容为JSON 格式，每个api处理函数的最后一个参数为返回内容的容器，默认为`resp`，可参考`app/api/example/example_kit.go`
```
resp["hello"] = "world"
return
```

### 如何生成api 返回值文档
可通过注释生成帮助文档, apidoc 会扫描`app/api` 中api实现, 并收集使用`resp` 中的内容以及`resp` 行的注释, 如下
注释中第一个字段表示当前返回值类型, `number`, `bool`, `string`, `object`等, 如果返回的为应用中定义的对象, 可以
通过关键字`@` 引用(注意该对象名必须唯一, 否则会混淆), 类型前加`[]` 表示返回数组, `@` 只会会搜索路径`ak/gw/model`,
`ak/orm/`以及`app/info` 中的对象

```
resp["intvalue"] = 1                                         // number number用与表示数值类型
resp["boolvalue"] = true                                     // bool bool用于表示布尔类型
resp["strvalue"] = "hello"                                   // string string 用于表示字符串类型
resp["objvalue"] = "object01"                                // object object 用于表示结构体对象类型
resp["arrayvalue"] = []string{"a", "b"}                      // []string 类型前加上[]用于表示数组类型
resp["user"] = u                                             // @Example 用户对象
resp["users"] = []interface{}{u}                             // @[]Example 用户对象
```

### 如何定义自定义数据类型接口
当前默认为JSON, 但是上传,下载或对接第三方系统时可能不是JSON, 所有会有自定义请求或返回值数据类型的需求, 定义使用proto=raw 即可
如
```
rawapi post none raw # 最后一个字段为raw
```

在接口中使用`req.Context` 即可操作原生接口如下
```
// application/json
// application/xml
// application/octet-stream

// get request body
body := req.Context.Request().Body
defer body.Close()

// echo data to resp

// method-01
// data, err := ioutil.ReadAll(body)
// CheckError(err)
// req.Context.Blob(http.StatusOK, "application/octet-stream", data)

// method-02
req.Context.Stream(http.StatusOK, "application/octet-stream", body)
```

### 如何定义应用命令
定义应用命令执行相关任务等, 如创建数据库索引, 同步数据

1. `cmd` 目录中新建一个`cmdname.go`, 参考`cmd/hello.go` 
2. 执行`go run . -appcmd` 进入appcmd
3. 执行`go run . -appcmd -hello` 执行hello命令

### 引用对象关系
1. Ref 引用关系，暂只支持单一引用，如一个user引用一个role (原因难以维护更新引用array) 如下user只能引用一个role

```
User
    name str
    role @Role
Role
    name str
```

2. 一个user如何有多个role? 定义一个关联对象UserRole

```
User
    name str
Role
    name str

UserRole
    user @User
    role @Role
```

### 管理对象关系
1. Mgr 管理关系，支持单一管理，对多管理，用于简化对象的数据结构, 如下user管理一个地址及多个订单

```
User
    name str
    address Address
        province str
        city str
        town str
        detail str
    order Order[]
        name str 
```

## 示例
简单的商城逻辑, 实现用户登录, 购买商品,评论等, 参考https://git.cloudhua.com/godev/gomall
