package helper

import (
	"errors"
	"fmt"
	"strings"
)

// ParseSort sort=a,-b -> [+a, -b]
func ParseSort(s string) (fileds []string) {
	_onece := make(map[string]bool)
	for _, filed := range strings.Split(s, ",") {
		if len(filed) == 0 {
			continue
		}

		if len(filed) == 1 && strings.ContainsAny(filed, "-+") {
			continue
		}

		if filed[0] == '-' || filed[0] == '+' {
			if _, exist := _onece[filed[1:]]; exist {
				continue
			}
			fileds = append(fileds, filed)
			_onece[filed[1:]] = true
		} else {
			if _, exist := _onece[filed]; exist {
				continue
			}
			fileds = append(fileds, fmt.Sprintf("+%s", filed))
			_onece[filed[1:]] = true
		}
	}
	return
}

var (
	filterToken   = []string{">=", "<=", "!=", "=", "<", ">"}
	filterMapping = map[string]string{
		">=": "$gte",
		"<=": "$lte",
		"!=": "$ne",
		"=":  "$eq",
		">":  "$gt",
		"<":  "$lt",
	}
)

// ParseFilter filter=status=0,price>=100,price<=300 -> {status: {"$eq": 0}, price: {"$gte": 100, "$lte"： 300}}
func ParseFilter(s string) (map[string]interface{}, error) {
	_m := make(map[string]interface{})
	for _, filed := range strings.Split(s, ",") {
		var items []string
		var token string
		for _, _token := range filterToken {
			items = strings.Split(filed, _token)
			if len(items) == 2 {
				token = _token
				break
			}
		}
		if items == nil || len(items) != 2 || token == "" {
			return nil, errors.New("invalid filter value")
		}

		key := items[0]
		op := filterMapping[token]
		value := items[1]
		fm, exists := _m[key]
		if !exists {
			fm = make(map[string]interface{})
		}
		fm.(map[string]interface{})[op] = value
		_m[key] = fm
	}
	return _m, nil
}
