package helper

import (
	. "gopher/ak/error"
	"net/http"
	"strconv"

	"github.com/labstack/echo"
)

func OKRequest(c echo.Context) error {
	return c.JSON(http.StatusOK, map[string]string{
		"stat":   "OK",
		"result": "ok",
	})
}

func OKRequestWith(c echo.Context, data map[string]interface{}) error {
	m := map[string]interface{}{
		"stat":   "OK",
		"result": "ok",
		"data":   data,
	}

	return c.JSON(http.StatusOK, m)
}

func RawOKRequest(c echo.Context) error {
	// do nothing in here
	return nil
}

func BadRequest(c echo.Context, message string) error {
	m := map[string]interface{}{
		"stat":    "ERROR",
		"result":  "error",
		"message": message,
	}
	return c.JSON(http.StatusBadRequest, m)
}

func QueryParamMustInt(c echo.Context, name string) int64 {
	value := c.QueryParam(name)
	Verify(value == "", ErrBadParams)
	valueInt, err := strconv.ParseInt(value, 10, 64)
	Verify(err != nil, ErrBadParams)
	return valueInt
}

func QueryParamMustString(c echo.Context, name string) string {
	value := c.QueryParam(name)
	Verify(value == "", ErrBadParams)
	return value
}
