package config

import (
	"fmt"
	"io/ioutil"
	"os"
	"strconv"

	"gopkg.in/yaml.v2"
)

type Config struct {
	Debug     bool   `yaml:"debug"`
	Name      string `yaml:"name"`
	LogDir    string `yaml:"logDir"`
	HTTPAddr  string `yaml:"httpAddr"`
	WebIndex  string `yaml:"webIndex"`
	WebStatic string `yaml:"webStatic"`
	DocStatic string `yaml:"docStatic"`
	MongoURL  string `yaml:"mongoURL"`
	RedisURL  string `yaml:"redisURL"`
	TokenName string `yaml:"tokenName"`
	File      struct {
		Type string `yaml:"type"`
		Home string `yaml:"home"`
	} `yaml:"file"`
	APIDOC struct {
		Title string `yaml:"title"`
		Desc  string `yaml:"desc"`
		URL   string `yaml:"url"`
	} `yaml:"apiDoc"`
}

func (c *Config) EnvInt(name string) int64 {
	value := os.Getenv(name)
	ival, err := strconv.ParseInt(value, 10, 64)
	if err != nil {
		return 0
	}
	return ival
}

func (c *Config) EnvMustInt(name string) int64 {
	value := os.Getenv(name)
	ival, err := strconv.ParseInt(value, 10, 64)
	if err != nil {
		panic(fmt.Sprintf("config env %s error %s", name, err.Error()))
	}
	return ival
}

func (c *Config) EnvStr(name string) string {
	return os.Getenv(name)
}

func (c *Config) EnvMustStr(name string) string {
	value, exist := os.LookupEnv(name)
	if !exist {
		panic(fmt.Sprintf("config env %s not found!", name))
	}
	return value
}

func (c *Config) EnvBool(name string) bool {
	value := os.Getenv(name)
	bval, err := strconv.ParseBool(value)
	if err != nil {
		return false
	}
	return bval
}

func (c *Config) EnvMustBool(name string) bool {
	value := os.Getenv(name)
	bval, err := strconv.ParseBool(value)
	if err != nil {
		panic(fmt.Sprintf("config env %s error %s", name, err.Error()))
	}
	return bval
}

var GlobalConfig *Config

// Init config
func Init(p string) {
	data, err := ioutil.ReadFile(p)
	if err != nil {
		panic(err.Error())
	}
	config := &Config{}
	err = yaml.Unmarshal(data, config)
	// err = json.Unmarshal(data, config)
	if err != nil {
		panic(err.Error())
	}
	GlobalConfig = config
}
