package main

import (
	"flag"
	"fmt"
	"gopher/ak/gw"
	"gopher/ak/orm"
	appcmd "gopher/cmd"
	"gopher/config"
	"gopher/kit"
	"os"

	"github.com/labstack/echo"
)

const (
	// put change log here
	// 0.0.1 create base service
	version = "0.0.1"
)

var (
	// Build information should only be set by -ldflags.
	BuildDate    string
	BuildGitHash string

	h       bool
	v       bool
	cfgpath string
	e       *echo.Echo
)

func showBuildInfo() {
	if len(BuildDate) > 0 {
		fmt.Println("BuildDate:", BuildDate)
		fmt.Println("BuildGitHash:", BuildGitHash)
	}
}

func cmd() {
	appcmd.NewAppCMD("config/dev.yaml", "-appcmd").Inject()
	flag.Bool("appcmd", false, "show appcmd and enter appcmd mode")
	flag.BoolVar(&h, "h", false, "this help")
	flag.BoolVar(&v, "v", false, "show version")
	flag.StringVar(&cfgpath, "c", "config/dev.yaml", "set cfg and start")
	flag.Parse()

	if v {
		fmt.Println("Version:", version)
		showBuildInfo()
		os.Exit(0)
	}

	if h {
		flag.Usage()
		os.Exit(0)
	}
}

func main() {
	// cmd
	cmd()

	// info
	showBuildInfo()

	// config
	config.Init(cfgpath)

	// kit
	kit.Init()

	// orm
	orm.Init(config.GlobalConfig.MongoURL)

	// gateway
	gw.Init()

	// this will block forever
	gw.StartAPP()
}
