package token

import (
	"time"

	"github.com/globalsign/mgo"
	"github.com/globalsign/mgo/bson"
)

type TokenAuthOnMongod struct {
	TokenName    string
	MongodURL    string
	TableName    string
	ExpiredAfter time.Duration
	mgoDB        *mgo.Database
	collection   *mgo.Collection
}

func NewTokenAuthOnMongod(mongodURL, tableName, tokenName string) TokenStore {
	mgoDB := getMGO(mongodURL)
	collection := mgoDB.C(tableName)

	// https://docs.mongodb.com/manual/tutorial/expire-data/#expire-documents-after-a-specified-number-of-seconds
	// The background task that removes expired documents runs every 60 seconds.
	err := collection.EnsureIndex(mgo.Index{Key: []string{"expiredAt"}, ExpireAfter: 1 * time.Second})
	if err != nil {
		panic(err)
	}

	return &TokenAuthOnMongod{
		TokenName:  tokenName,
		MongodURL:  mongodURL,
		TableName:  tableName,
		mgoDB:      mgoDB,
		collection: collection,
	}
}

func (s *TokenAuthOnMongod) Add(key string, uid interface{}, expiredAfter time.Duration) (err error) {
	tok := &token{Key: key, UID: uid, ExpiredAt: bson.Now().Add(expiredAfter)}
	return s.collection.Insert(tok)
}

func (s *TokenAuthOnMongod) Get(key string) (uid interface{}, err error) {
	tok := &token{}
	err = s.collection.FindId(key).One(&tok)
	if err != nil && err.Error() == "not found" {
		return nil, nil
	}
	// The background task that removes expired documents runs every 60 seconds.
	if bson.Now().After(tok.ExpiredAt) {
		return nil, nil
	}
	uid = tok.UID
	return
}

func getMGO(url string) *mgo.Database {
	opts, err := mgo.ParseURL(url)
	if err != nil {
		panic(err)
	}
	session, err := mgo.DialWithInfo(opts)
	if err != nil {
		panic(err)
	}

	return session.DB(opts.Database)
}
